# -*- coding: utf-8 -*-
"""
Created on Tue Apr 23 17:09:03 2019

@author: z5025317
"""

#==========================================================#
#Load packages
#==========================================================#
import geopandas as gpd
import numpy as np
import os
import pandas as pd
import glob
import matplotlib
import matplotlib.pyplot as plt
from datetime import datetime
from datetime import timedelta
from matplotlib.backends.backend_pdf import PdfPages
from ggplot import *

#name of scenario
Plot_scenario = 'HD_grid_V1'
ylims = [0,40]
variable = 'salinity'

#set directory path for output files
output_directory = 'H:/WRL_Projects/Hunter_CC_Modeling/Module_6/03_Results/Output/Postprocessed/' +  Plot_scenario + '/Figures/'
if not os.path.exists(output_directory):
    os.makedirs(output_directory)
    
#input csv with species and their salinity optima and thresholds
salinity_eco_csv = "H:/WRL_Projects/Hunter_CC_Modeling/Module_6/03_Results/Eco_thresholds/Salintiy_Hunter_physiology_V1.csv"
salinity_eco_df = pd.read_csv(salinity_eco_csv, index_col=2)
salinity_eco_df = salinity_eco_df[salinity_eco_df.index.notnull()]

#load and refine node vs mesh shapefile to store the results of the statistics
test = gpd.read_file('H:/WRL_Projects/Hunter_CC_Modeling/Module_6/03_Results/GIS_output/Hunter_Saltmarsh_salinity_stats_V2_clipped_estuary.shp')
tost = test.copy()
tost= tost.to_crs({'init': 'epsg:3857'})
print tost.crs
tost.head(2)

#Now the area in square kilometers
tost["area"] = tost['geometry'].area/ 10**6

#calculate areas of salinity intervals at 10th of the total range

Area_df = {}
Area_f_df = {}
incl_bool=False
Nr_of_intervs = 100
for i in range(1,Nr_of_intervs+1,1):
    Interval = ((tost['0_med'].max() - tost['0_med'].min())/Nr_of_intervs)
    if i == 1:
        Area_df[round(Interval*i - Interval/2 ,2)] = round(tost[tost['0_med'].between(0,Interval, inclusive=incl_bool)]['area'].sum(),2)
        Area_f_df[round(Interval*i - Interval/2 ,2)] = round(tost[tost['9_med'].between(0,Interval, inclusive=incl_bool)]['area'].sum(),2)
    else:
        Area_df[round(Interval*i - Interval/2 ,2)] = round(tost[tost['0_med'].between(Interval*(i-1), i*Interval, inclusive=incl_bool)]['area'].sum(),2)
        Area_f_df[round(Interval*i - Interval/2 ,2)] = round(tost[tost['9_med'].between(Interval*(i-1), i*Interval, inclusive=incl_bool)]['area'].sum(),2)

Area_df = pd.DataFrame.from_dict(Area_df, orient='index')
Area_df.columns = ['0_med']

Area_f_df2 = pd.DataFrame.from_dict(Area_f_df, orient='index')
Area_f_df2.columns = ['0.9_med']

SLR_stats =  pd.concat([Area_df, Area_f_df2], axis=1, join='outer')
SLR_stats = SLR_stats.sort_index(axis=0, level=None, ascending=True)

SLR_stats.plot()


png_out_name = output_directory +'/' +  variable + '_salinity_habitat_analysis_' + Plot_scenario + '.pdf' 
fig = plt.figure(figsize=(15,8))
ax=plt.subplot(1,1,1) 
plt.title('Salinity range habitat size for Hunter River 0 and 0.9m sea level rise') 
SLR_stats.plot( legend=False, ax=ax)
plt.ylabel("Area [square km]")
plt.xlabel("Salinity [PSU]")
fig.tight_layout()
fig.savefig(png_out_name)
plt.close()








