# -*- coding: utf-8 -*-
#==========================================================#
#Last Updated - June 2018
#@author: z5025317 Valentin Heimhuber
#code for creating climate prioritization plots for NARCLIM variables. 
#Inputs: Uses CSV files that contain all 12 NARCLIM model runs time series for 1 grid cell created with: P1_NARCliM_NC_to_CSV_CCRC_SS.py

#==========================================================#
#Load packages
#==========================================================#
import geopandas as gpd
import numpy as np
import os
import pandas as pd
import glob
import matplotlib
import matplotlib.pyplot as plt
from datetime import datetime
from datetime import timedelta
from matplotlib.backends.backend_pdf import PdfPages
from ggplot import *
matplotlib.style.use('ggplot')

#==========================================================#
#Input parameters
#==========================================================#
#set scenario codes and beginning and end years and corresponding scenario code
fs=['Hwq003', 'Hwq005']
scenario_real_names = ['Present', 'Far_Future']
HDvariables = ['depth', 'elev']
variable = 'elev' #depth or vel

#Save bathyfile?
save_bathyfile = False

#set max rows to be read in from the RMA2 csv files
maxrowsincsv = 15000

#name of scenario
Plot_scenario = 'HD_grid_V1'
ylims = [-1.1,8]

#time frames for analysis
startdate =  '1999 01 01'
enddate = '1999 12 01'
tideanalysis_day = '1999 04 26'
#year=range(startyear, endyear+1)

#load and refine node vs mesh shapefile to store the results of the statistics
Node_mesh = gpd.read_file('H:/WRL_Projects/Hunter_CC_Modeling/Module_6/03_Results/GIS_data/Mesh_Node_GIS_Setup/hcc002_fishnet_500m_V3_joinXY.shp')
Node_mesh.index =  Node_mesh['Field1']
Node_mesh = Node_mesh.iloc[:,-4:]
Node_mesh.columns = ['Node', 'X', 'Y', 'geometry']
node = Node_mesh['Node'].values
chainages = node

#load actual mesh file
Actual_mesh = gpd.read_file('H:/WRL_Projects/Hunter_CC_Modeling/Module_6/03_Results/GIS_data/Mesh_Node_GIS_Setup/hcc002.shp')

#set directory path for output files
output_directory = 'H:/WRL_Projects/Hunter_CC_Modeling/Module_6/03_Results/Output/Postprocessed/' +  Plot_scenario + '/Figures'

#set input directories and data
#tidal boundary data
tides_csv='C:/Users/z5025317/OneDrive - UNSW/Hunter_CC_Modeling/07_Modelling/01_Input/Tide Generator/Tidal Simulation Python Script (BMM)/Tides_1990_2010.csv'
#CSV file of mesh nodes and corresponding chainages
nodes_csv = 'H:/WRL_Projects/Hunter_CC_Modeling/Module_6/03_Results/Chainages/Hunter_nodes.csv' 
#River flow boundary
Flow_csv = 'H:/WRL_Projects/Hunter_CC_Modeling/Module_6/01_Input/BCGeneration/Scenarios/Calibration/Calibration_Greta.csv' 
#==========================================================#

#==========================================================#
#set plot parameters
ALPHA_figs = 1
font = {'family' : 'sans-serif',
        'weight' : 'normal',
        'size'   : 14}
matplotlib.rc('font', **font)
#==========================================================#

#==========================================================#
#output_directory = 'J:/Project wrl2016032/NARCLIM_Raw_Data/Extracted'
if not os.path.exists(output_directory):
    os.makedirs(output_directory)
    print("output directory folder didn't exist and was generated")
#==========================================================#
    

#========================================================#
#automated part of the code doing the data extraction
#==========================================================

#Load boundary flow data
Flow_df = pd.read_csv(Flow_csv, parse_dates=True, dayfirst=True, index_col=0)['Q[ML/d]']
Flow_df.columns = ['Q[ML/d]']
Flow_df = Flow_df[datetime.strptime(startdate , '%Y %m %d').date():datetime.strptime(enddate, '%Y %m %d').date()]
Flow_df.plot()
#Load tide boundary data
tides_dfm = pd.read_csv(tides_csv, parse_dates=True, index_col=0)
tides_dfm.columns = ['tide']

#read csv file with extracted RMA data
#RMA_df = pd.read_csv(RMA_data_csv, parse_dates=True, index_col=0)

#load RMA2 data
HD_Summary_df = pd.DataFrame() 
for variable in HDvariables:
    Summary_df = pd.DataFrame() 
    df = pd.DataFrame() 
    for f in fs:
        #set input and output directories
        input_directory = 'H:/WRL_Projects/Hunter_CC_Modeling/Module_6/03_Results/Output/' +  f 
        # Set working direcotry (where postprocessed NARClIM data is located)
        os.chdir(input_directory)
        #==========================================================#
        #Load data file
        if variable == 'depth' or variable == 'elev':
            Clim_Var_CSVs = glob.glob('*' + variable + '*')
            clim_var_csv_path = Clim_Var_CSVs[0]
            df = pd.read_csv(clim_var_csv_path, index_col=False,  sep=' ', nrows= maxrowsincsv)
            df.index = pd.to_datetime(df.Year, format = '%Y') + pd.to_timedelta(df.Hour, unit='h')
            df= df.drop(columns=['Year', 'Hour'])
            a=len(df.columns)-1
            df=df.iloc[:,:a]
        if variable == 'vel':
            #x velocity
            Clim_Var_CSVs = glob.glob('*' +'x'+ variable + '*')
            clim_var_csv_path = Clim_Var_CSVs[0]
            df = pd.read_csv(clim_var_csv_path, index_col=False,  sep=' ', nrows= maxrowsincsv)
            df.index = pd.to_datetime(df.Year, format = '%Y') + pd.to_timedelta(df.Hour, unit='h')
            dfx= df.drop(columns=['Year', 'Hour','1'])
            #y velocity
            Clim_Var_CSVs = glob.glob('*' +'y'+ variable + '*')
            clim_var_csv_path = Clim_Var_CSVs[0]
            df = pd.read_csv(clim_var_csv_path, index_col=False,  sep=' ',nrows= maxrowsincsv)
            df.index = pd.to_datetime(df.Year, format = '%Y') + pd.to_timedelta(df.Hour, unit='h')
            dfy= df.drop(columns=['Year', 'Hour','1'])
            df = np.sqrt(dfx*dfx + dfy*dfy)    
        df.columns = df.columns + '_'+ f + '_'+ variable
        #cut down the df to start and end date
        df = df[datetime.strptime(startdate , '%Y %m %d').date():datetime.strptime(enddate, '%Y %m %d').date()]
        Summary_df = pd.concat([Summary_df, df], axis=1, join='outer')
    HD_Summary_df  = pd.concat([HD_Summary_df , Summary_df], axis=1, join='outer')        

     
#Summary_df.columns = chainages


#subset the input data frame
Present_df = HD_Summary_df.filter(regex='Hwq003').filter(regex='elev')
Far_future_df = HD_Summary_df.filter(regex='Hwq005').filter(regex='elev')

#Create Bathymetry along the nodes and chainage

Elev_df = HD_Summary_df.filter(regex='Hwq003').filter(regex='elev')
Elev_df.columns = node
Depth_df = HD_Summary_df.filter(regex='Hwq003').filter(regex='depth')
Depth_df.columns = node
Bathy_df = pd.DataFrame(Elev_df.loc[Elev_df.index[0]] -  Depth_df.loc[Elev_df.index[0]])
Bathy_df.columns = ['Bathymetry']
if save_bathyfile:
    bathy_out_path = output_directory + 'Hunter_nodes_bathy.shp'
    Node_mesh_df.to_file(bathy_out_path, driver='ESRI Shapefile') 


#plot bathy
Node_mesh_df  = pd.concat([Node_mesh , Bathy_df], axis=1, join='outer')  
png_out_name = output_directory + 'bathy.pdf'
#plot figure
fig = plt.figure(figsize=(12,17))
ax=plt.subplot(1,1,1) 
Node_mesh_df.plot(column='Bathymetry',
                         legend=True,
                         cmap='viridis',
                         ax=ax,
                         vmin=-20, vmax=0);
Actual_mesh.plot(ax=ax, alpha=1, edgecolor='grey',facecolor="None")
plt.title('Bathymetry of the Hunter River estuary') 
ax.legend()     
fig.tight_layout()
fig.savefig(png_out_name)
plt.close()


#==========================================================#
#Tidal analysis
#==========================================================#
#read in tide data
tides_df = tides_dfm[datetime.strptime(tideanalysis_day , '%Y %m %d').date()- timedelta(days=2) : datetime.strptime(tideanalysis_day , '%Y %m %d').date() + timedelta(days=3) ]
central_tides_df = tides_dfm[datetime.strptime(tideanalysis_day , '%Y %m %d').date():datetime.strptime(tideanalysis_day , '%Y %m %d').date()+ timedelta(days=1) ]
maxtide_index = central_tides_df['tide'].idxmax()
end_maxtide_index =  maxtide_index + timedelta(days=0.4)
beg_maxtide_index =  maxtide_index - timedelta(days=0.2)
short_tides_df = tides_df[beg_maxtide_index: end_maxtide_index]
short_tides_df.plot(cmap='viridis',  legend=True)
mintide_index = short_tides_df['tide'].idxmin()

#plot the minimum and maximum elevation along the estuary occuring within a single tidal cycle
Short_tide_elev_df = HD_Summary_df.filter(regex='elev')[min(short_tides_df.index):max(short_tides_df.index)]
max_min_1cycle_elev_0slr = pd.concat([Short_tide_elev_df.filter(regex=fs[0]).max(), Short_tide_elev_df.filter(regex=fs[0]).min()], axis=1, join='inner')  
max_min_1cycle_elev_09slr = pd.concat([Short_tide_elev_df.filter(regex=fs[1]).max(), Short_tide_elev_df.filter(regex=fs[1]).min()], axis=1, join='inner')
max_min_1cycle_elev_0slr.index = chainages
max_min_1cycle_elev_09slr.index = chainages
max_min_1cycle_elev = pd.concat([max_min_1cycle_elev_0slr, max_min_1cycle_elev_09slr], axis=1, join='outer')
max_min_1cycle_elev.columns = ['MaxElev 0 SLR','MinElev 0 SLR','MaxElev 0.9 SLR','MinElev 0.9 SLR' ]
max_min_1cycle_elev  = pd.concat([Node_mesh , max_min_1cycle_elev], axis=1, join='outer')  
#max_min_1cycle_elev.plot(cmap='viridis_r', column='MaxElev 0.9 SLR' , legend=True,  vmin=-1, vmax=2)


#plot the elevation along the estuary at exactly the time of high and low tide
max_min_extime_elev_0slr = pd.concat([HD_Summary_df.filter(regex=fs[0]).filter(regex='elev').loc[maxtide_index], Summary_df.filter(regex=fs[0]).loc[mintide_index]], axis=1, join='outer')
max_min_extime_elev_09slr = pd.concat([HD_Summary_df.filter(regex=fs[1]).filter(regex='elev').loc[maxtide_index], Summary_df.filter(regex=fs[1]).loc[mintide_index]], axis=1, join='outer')
max_min_extime_elev_0slr.index = chainages
max_min_extime_elev_09slr.index = chainages
max_min_extime_elev = pd.concat([max_min_extime_elev_0slr, max_min_extime_elev_09slr], axis=1, join='outer')
max_min_extime_elev.columns = ['HigT Elev 0SLR','LowT Elev 0SLR','HigT Elev 0.9 SLR','LowT Elev 0.9 SLR' ]
max_min_extime_elev  = pd.concat([Node_mesh , max_min_extime_elev], axis=1, join='outer')  
#max_min_extime_elev.plot(cmap='viridis_r', column='HigT Elev 0.9 SLR' , legend=True,  vmin=-1, vmax=2)
#==========================================================#
#End of Tidal analysis
#==========================================================#


#==========================================================#
#Plot Tidal analysis
#==========================================================#
ylims = [-0.6, 1]
png_out_name =  output_directory + variable + '_high_low_tide_analysis_' + Plot_scenario + '.pdf' 
fig = plt.figure(figsize=(23,20))
ax=plt.subplot(2,2,1) 
plt.title('High tide water surface elevation along the estuary') 
#max_min_extime_elev.plot(cmap='viridis_r', column='HigT Elev 0SLR' , legend=True, vmin=ylims[0], vmax=ylims[1],ax=ax)
max_min_extime_elev.plot(cmap='viridis_r', column='HigT Elev 0SLR' , legend=True,ax=ax)
#lgd = ax.legend(loc='center left', bbox_to_anchor=(1, 0.5))  
ax.legend()                    
#plot tides
ax=plt.subplot(2,2,2)
plt.title('Low tide water surface elevation along the estuary') 
max_min_extime_elev.plot(cmap='viridis_r', column='LowT Elev 0SLR' , legend=True,ax=ax)
#lgd = ax.legend(loc='center left', bbox_to_anchor=(1, 0.5))   
ax.legend()    
#plot tides
ax=plt.subplot(2,1,2)
plt.ylim(ylims)
plt.title('Tide at open ocean boundary') 
short_tides_df.plot(cmap='viridis',  legend=False, ax=ax)
ax.axvline(maxtide_index, color='black', linestyle='--')
ax.axvline(mintide_index, color='black', linestyle='-')
plt.xlabel("Time")
plt.ylabel("Water level [m AHD]")
#lgd = ax.legend(loc='center left', bbox_to_anchor=(1, 0.5)) 
ax.legend()     
fig.tight_layout()
fig.savefig(png_out_name)
plt.close()


#==========================================================#
#animate 2 tidal cycles baseline scenario:
#==========================================================#
Six_day_tide_elev_df = HD_Summary_df.filter(regex=fs[0]).filter(regex='elev')[maxtide_index - timedelta(days=2): maxtide_index + timedelta(days=2)]
Six_day_tide_elev_df.columns = chainages
Six_day_tides_df = tides_df[maxtide_index - timedelta(days=2): maxtide_index + timedelta(days=2)]
Six_day_tides_df.plot(cmap='viridis',  legend=True)

Six_day_flow_df = Flow_df[maxtide_index - timedelta(days=2): maxtide_index + timedelta(days=2)]
Six_day_flow_df = pd.concat([Six_day_tides_df,  Six_day_flow_df], axis=1, join='outer')['Q[ML/d]'].interpolate()
Six_day_flow_df.plot(cmap='viridis',  legend=True)

ylims = [-0.6, 1]
ii=0
for index in Six_day_tide_elev_df.index[0::4]:
    ii=ii+1
    #index = Six_day_tide_elev_df.index[1]
    png_out_path =  output_directory +  '/TidalCycle_' + variable + '_' + Plot_scenario +  '/'
    if not os.path.exists(png_out_path):
        os.makedirs(png_out_path)
    #png_out_name = png_out_path + str(index) + 'longitudinalXS.png' 
    png_out_name = png_out_path + str(ii) + variable +  '_longitudinalXS.png' 
    
    fig = plt.figure(figsize=(17,20))
    ax=plt.subplot(2,1,1) 
    plt.title('Water surface elevation along the estuary') 
    df1 = pd.DataFrame(Six_day_tide_elev_df.loc[index])
    df1.columns = ['Instant']
    Six_day_tide_elev_df_instant  = pd.concat([Node_mesh , df1], axis=1, join='outer')  
    Six_day_tide_elev_df_instant.columns
    Six_day_tide_elev_df_instant.plot(cmap='viridis_r', column='Instant' ,vmin=ylims[0], vmax=ylims[1],  legend=True,ax=ax)
    #plot tides
    ax=plt.subplot(2,1,2)
    plt.ylim(ylims)
    plt.title('Tide at open ocean boundary') 
    Six_day_tides_df.plot(cmap='viridis',  legend=False, ax=ax)
    ax.axvline(index, color='green', linestyle='--')
    plt.xlabel("Time")
    plt.ylabel("Water level [m AHD]")

    fig.tight_layout()
    fig.savefig(png_out_name)
    plt.close()

        
        



#==========================================================#
#animate 2 tidal cycles baseline + SLR scenario:
#==========================================================#
Six_day_tide_elev_df_0slr = HD_Summary_df.filter(regex=fs[0]).filter(regex='elev')[maxtide_index - timedelta(days=1): maxtide_index + timedelta(days=1)]
Six_day_tide_elev_df_0slr.columns = chainages
Six_day_tide_elev_df_09slr = HD_Summary_df.filter(regex=fs[1]).filter(regex='elev')[maxtide_index - timedelta(days=1): maxtide_index + timedelta(days=1)]
Six_day_tide_elev_df_09slr.columns = chainages
Six_day_tides_df = tides_df[maxtide_index - timedelta(days=1): maxtide_index + timedelta(days=1.5)]
Six_day_tides_df.plot(cmap='viridis',  legend=True)

ii=0
for index in Six_day_tide_elev_df_0slr.index:
    ii=ii+1
    #index = Six_day_tide_elev_df_0slr.index[1]
    png_out_path =  output_directory + '/' + Plot_scenario +   '/TidalCycle_SLR_' + variable + '_' + Plot_scenario +   '/'
    if not os.path.exists(png_out_path):
        os.makedirs(png_out_path)
    
    #png_out_name = png_out_path + str(index) + 'longitudinalXS.png' 
    png_out_name = png_out_path + str(ii) + variable +  '_longitudinalXS.png' 
    max_min_extime_elev_both = pd.concat([Six_day_tide_elev_df_0slr.loc[index],  Six_day_tide_elev_df_09slr.loc[index]], axis=1, join='outer')
    max_min_extime_elev_both.columns = ['Present Day','0.9m SLR']
    
    fig = plt.figure(figsize=(17,12))
    ax=plt.subplot(2,1,1) 
    plt.ylim(ylims)
    plt.title('Water surface elevation along the estuary') 
    ax.set_prop_cycle(color=['slateblue','tomato'], linestyle=['-',  '-'], lw=[2,2])
    max_min_extime_elev_both.plot(ax=ax)
    ax.legend(loc=1)
    plt.axhline(y=0, color='r', linestyle='--', lw=1, alpha=0.5)  
    plt.ylim(-1.1,2.5)
    plt.ylabel("Water level [m AHD]")
    plt.xlabel("Distance from ocean boundary [km]")
    
    #plot tides
    ax=plt.subplot(2,1,2)
    plt.ylim(-1.1,1.5)
    plt.title('Tide at open ocean boundary') 
    Six_day_tides_df.plot(cmap='viridis',  legend=False, ax=ax)
    ax.axvline(index, color='green', linestyle='--')
    plt.xlabel("Time")
    plt.ylabel("Water level [m AHD]")
    fig.tight_layout()
    fig.savefig(png_out_name)
    plt.close()
 
        
































#for NODE in node:
#    NODE = str(NODE)
#    #=========#
#    #comparison time series plot at node
#    #=========#
#    #out name
#    png_out_file_name = comparison_code + '_' +  variable  + '_' + NODE + '_time_series.png'
#    png_out_path = png_output_directory + '/' + png_out_file_name
#    #
#    plotin_df = Summary_df.filter(regex=NODE)
#    #prepare data frame for plot
#    fig = plt.figure(figsize=(14,18))
#    ax=plt.subplot(4,1,1) 
#    #start and end dat of plots
#    plt.title(variable  + '_' + NODE + ' - time series')
#    start_date = datetime(int('1996'), int('02'), int('01'))
#    end_date = datetime(int('1996'), int('02'), int('5'))
#    plotin_df1=plotin_df[start_date:end_date]
#    ymin = min(plotin_df1.min()) - 0.01 *min(plotin_df1.min())
#    ymax = max(plotin_df1.max()) + 0.01 *max(plotin_df1.max())  
#    plotin_df1.plot(ylim=(ymin,ymax) ,legend=False, ax=ax) 
#    #plt.legend( loc=1)    
#    plt.xticks([])
#    
#    ax=plt.subplot(4,1,2) 
#    start_date = datetime(int('1996'), int('02'), int('01'))
#    end_date = datetime(int('1996'), int('02'), int('3'))
#    plotin_df2=plotin_df[start_date:end_date]
#    ymin = min(plotin_df2.min()) - 0.1 *min(plotin_df2.min())
#    ymax = max(plotin_df2.max()) + 0.1 *max(plotin_df2.max()) 
#    plotin_df2.plot(ylim=(ymin,ymax),legend=False, ax=ax)
#    #plt.legend( loc=1)
#    ax.xaxis.grid(False)
#    
#    ax=plt.subplot(4,1,3)
#    plt.title(variable  + '_' + NODE + ' - monthly maximum')
#    Monthly_max = plotin_df.resample('M').max()
#    Monthly_max.plot(ax=ax,legend=False)
#    plt.legend( loc=1)
#    ax.xaxis.grid(False)
#    
#    ax=plt.subplot(4,1,4)
#    plt.title(variable  + '_' + NODE + ' - monthly mean')
#    Monthly_mean = plotin_df.resample('M').mean()
#    Monthly_mean.plot(ax=ax)
#    #plt.legend(bbox_to_anchor=(0., 1.02, 1., .102), loc=3,
#    #   ncol=2, mode="expand", borderaxespad=0.)
#    #plt.legend( loc=1)
#    ax.xaxis.grid(False)
#
#    #ax.patch.set_alpha(ALPHA_figs)
#    fig.patch.set_alpha(ALPHA_figs)
#    fig.tight_layout()
#    fig.savefig(png_out_path)
#    plt.close()
#    #=========#



