# -*- coding: utf-8 -*-
#==========================================================#
#Last Updated - June 2018
#@author: z5025317 Valentin Heimhuber
#code for creating climate prioritization plots for NARCLIM variables. 
#Inputs: Uses CSV files that contain all 12 NARCLIM model runs time series for 1 grid cell created with: P1_NARCliM_NC_to_CSV_CCRC_SS.py

#Datum offset factors can be found at https://www.mhl.nsw.gov.au/docs/Data%20Conditions%20and%20Fit%20for%20Purpose.pdf
#==========================================================#

#==========================================================#
#Load packages
#==========================================================#
import numpy as np
import os
import pandas as pd
import geopandas as gpd
import glob
import matplotlib
import matplotlib.pyplot as plt
from datetime import datetime
from datetime import timedelta
#from matplotlib.backends.backend_pdf import PdfPages
import seaborn
matplotlib.style.use('ggplot')
#set plot parameters
ALPHA_figs = 1
font = {'family' : 'sans-serif',
        'weight' : 'normal',
        'size'   : 14}
matplotlib.rc('font', **font)
#==========================================================#




#==========================================================#
#Set Input parameters
#==========================================================#
#set scenario codes and beginning and end years and corresponding scenario code
River = 'Hastings'
Order_ID = 'Core_order' #choose order column name

#input scenarios
fs= ['HAS_CAL_01' , 'HAS_CAL_02'] #,'TWD_HYD_CAL_12'] #['TWD_HYD_CAL_05',  'TWD_HYD_CAL_06'] #['TWD_HYD_CAL_01','TWD_HYD_CAL_02']#, 'SLR_1m_2008_FN1']
#scenario_real_names = ['TWD_HYD_CAL_03',  'TWD_HYD_CAL_04'] #['TWD_HYD_CAL_01', 'TWD_HYD_CAL_02']   #'2008_RMA_1m_SLR'] #these will be used in the plotting so weird scenarios codes can be replaced with more intuitive names if desired
scenario_real_names = fs
scenario_dict = dict(zip(fs, scenario_real_names))

#name of plot scenario/experiment . This is a unique identifier for this particular scenario comparison
Plot_scenario =  fs[0] +'_vs_' + fs[1] + 'v2'#'Cal_05vs06vsMHL'

#HDvariables = ['depth', 'elev']
variable = 'elev' #depth or vel

maxrowsincsv = 30000000 #limits the nr. of timesteps read in via the RMA extracted txt files
ylims = [-1,1.7] #usually the min and max observed water levels in the estuary in AHD used for plot ylims

#time frames for analysis
startdate =  '2017 01 01'
enddate = '2017 03 30'
tideanalysis_day = '2017 02 01'
#year=range(startyear, endyear+1)
#========================================================#


#========================================================#
#mostly automated part of the code but might have to be double checked - debugged
#========================================================#
#set directory path for input and output files
main_input_directory = 'J:/Project/wrl2018064 Fisheries RAP/04_Working/05_Modelling/RMA/HEMIP/Models/' + River + '/04_Results/Output/'
output_directory = 'J:/Project/wrl2018064 Fisheries RAP/04_Working/05_Modelling/RMA/HEMIP/Models/' + River + '/04_Results/Output/' + '/Postprocessed'
if not os.path.exists(output_directory + '/' + Plot_scenario):
    os.makedirs(output_directory+ '/' + Plot_scenario)

#load and refine node vs mesh shapefile to store the results of the statistics    
Gauge_exchange_df =  pd.read_csv(glob.glob(main_input_directory + fs[0] + '_' + Order_ID + '/*.csv' )[0])  
keys = Gauge_exchange_df['NrstNode'].values
gauges = Gauge_exchange_df['Station Na'].values
#node_dict = dict(zip(keys, gauges))
order = Gauge_exchange_df[Order_ID].astype(int).values
order_dict = dict(zip(gauges, order))

#load and refine node vs mesh shapefile to store the results of the statistics    
Gauge_exchange_df =  pd.read_csv(glob.glob(main_input_directory + fs[1] + '_' + Order_ID + '/*.csv' )[0])  
keys2 = Gauge_exchange_df['NrstNode'].values
gauges2 = Gauge_exchange_df['Station Na'].values
keys =  np.append(keys, keys2)
gauges = np.append(gauges, gauges2)
node_dict = dict(zip(keys, gauges))


#Load in situ gauge data
WL_df =  pd.DataFrame()
for gauge in gauges:
    #gauge = gauges[1]
    #gauge = 'Harrington'
    print ('loading MHL gauge data for ' + gauge)
    CSVpath = glob.glob('J:/Project/wrl2018064 Fisheries RAP/04_Working/05_Modelling/RMA/HEMIP/Global_Data/MHL_Gauge_Data/20200409_Recieved/*/' + '*' +  gauge.replace(" ", "") + '*' )
    Full_df = pd.read_csv(CSVpath[0] ,header= 0, skiprows= 30, parse_dates = True) 
    Full_df.index = pd.to_datetime(Full_df.Date, format = '%d/%m/%Y') + pd.to_timedelta(Full_df.Time, unit='h') 
    #Full_df = Full_df.loc[datetime.strptime(startdate , '%Y %m %d').date()  : datetime.strptime(enddate , '%Y %m %d').date()]
    Full_df = Full_df.iloc[:,[-2]]  
    Full_df.columns = [gauge]  
    Full_df = Full_df.loc[~Full_df.index.duplicated(keep='first')]  #get rid of possible duplicate dates 
    Full_df = Full_df.replace(to_replace ='---', value =np.NaN) #--- is used as the no data value in the MHL data provided
    Full_df  = pd.DataFrame(pd.to_numeric(Full_df[gauge]))
    Full_df = Full_df  - np.float(Gauge_exchange_df['AHDadjust'][Gauge_exchange_df['Station Na']==gauge])   #subtract the datum adjustment factor provided by MHL as written in the shapefile
    WL_df = pd.concat([WL_df, Full_df], axis=1)
#postprocess the in situ gauge dataframe       
#WL_df = WL_df.replace(to_replace ='---', value =np.NaN) #--- is used as the no data value in the MHL data provided
#WL_df = WL_df.convert_objects(convert_numeric=True)
#WL_df = WL_df - datum_offset  
order_colnames=[]
for colname in WL_df.columns:
    print (colname)
    order_colnames.append(str(order_dict[colname])  + '_' + colname)
#WL_df.columns = order_colnames
WL_df.columns = [str(s) + '_MHL' for s in order_colnames]
order_colnames.sort() #this will late be used to loop through the MHL gauges in a downstream to upstream order
order_colnames = pd.Series(order_colnames).str.replace("(",'')
order_colnames = pd.Series(order_colnames).str.replace(")",'')
#========================================================#



#========================================================#
#fully automated part of the code doing the data extraction
#========================================================#
#load RMA2 data
Summary_df = pd.DataFrame() 
df = pd.DataFrame() 
for f in fs:
    #set input and output directories
    input_directory = main_input_directory +  f + '_' + Order_ID
    # Set working direcotry (where postprocessed NARClIM data is located)
    os.chdir(input_directory)
    #Load data file
    if variable == 'depth' or variable == 'elev':
        Clim_Var_CSVs = glob.glob('*' + variable + '*')
        clim_var_csv_path = Clim_Var_CSVs[0]
        df = pd.read_csv(clim_var_csv_path, index_col=False,  sep=' ', nrows= maxrowsincsv)
        df.index = pd.to_datetime(df.Year, format = '%Y') + pd.to_timedelta(df.Hour, unit='h')
        df= df.drop(columns=['Year', 'Hour'])
        a=len(df.columns)-1
        df=df.iloc[:,:a]
    if variable == 'vel':
        #x velocity
        Clim_Var_CSVs = glob.glob('*' +'x'+ variable + '*')
        clim_var_csv_path = Clim_Var_CSVs[0]
        df = pd.read_csv(clim_var_csv_path, index_col=False,  sep=' ', nrows= maxrowsincsv)
        df.index = pd.to_datetime(df.Year, format = '%Y') + pd.to_timedelta(df.Hour, unit='h')
        dfx= df.drop(columns=['Year', 'Hour','1'])
        #y velocity
        Clim_Var_CSVs = glob.glob('*' +'y'+ variable + '*')
        clim_var_csv_path = Clim_Var_CSVs[0]
        df = pd.read_csv(clim_var_csv_path, index_col=False,  sep=' ',nrows= maxrowsincsv)
        df.index = pd.to_datetime(df.Year, format = '%Y') + pd.to_timedelta(df.Hour, unit='h')
        dfy= df.drop(columns=['Year', 'Hour','1'])
        df = np.sqrt(dfx*dfx + dfy*dfy)    
    df.columns = df.columns + '_'+ f + '_'+ variable
    #cut down the df to start and end date
    df = df[datetime.strptime(startdate , '%Y %m %d').date():datetime.strptime(enddate, '%Y %m %d').date()]
    Summary_df = pd.concat([Summary_df, df], axis=1, join='outer')      
HD_Summary_df =     Summary_df 
#========================================================#
    




#========================================================#
#interactive data analysis part of the code - alternatively, the dataframe could be written to csv at this stage and analysed with a separate code. 
#========================================================#   
#Summary_df.columns = chainages
#subset the input data frame
Summary_df_nodes = pd.DataFrame() 
#for node, gauges in nodes: 
for key,value in node_dict.items():
    df1 = HD_Summary_df.filter(regex='^'+ str(key)+'_').filter(regex=variable)
    df1.columns = [value + '_' + str(s) for s in df1.columns]
    Summary_df_nodes = pd.concat([Summary_df_nodes, df1], axis=1, join='outer')

for key,value in scenario_dict.items():
    Summary_df_nodes.columns = pd.Series(Summary_df_nodes.columns).str.replace(key, value)
    
combined_df  = pd.DataFrame() 
combined_df  = pd.concat([Summary_df_nodes , WL_df], axis=1, join='outer') 
combined_df.columns = pd.Series(combined_df.columns).str.replace("(",'') #brackets mess up the regex commands below
combined_df.columns = pd.Series(combined_df.columns).str.replace(")",'')
combined_df_hourly = combined_df.resample('1H').max()

#this needs to be fixed to be more generic! 
Histogram_df = combined_df_hourly.loc[datetime.strptime(startdate , '%Y %m %d').date()  : datetime.strptime(enddate , '%Y %m %d').date()] #dataframe for histogram comparison between RMA and MHL
#========================================================#   
#drop first gauge which doesn't have any data in 2006
#gauges = gauges[1:]




#========================================================#   
# Plot water level histograms at control points
#========================================================#   
ylims = [-0.9,2.5]
#set plot parameters
ALPHA_figs = 1
font = {'family' : 'sans-serif',
        'weight' : 'normal',
        'size'   : 6}
matplotlib.rc('font', **font)

i=1
png_out_name = output_directory + '/' + Plot_scenario + '/RMA_vs_MHL_gauges_water level histogram and exceedance planes '+ startdate + '_' + enddate + '.pdf' 
fig = plt.figure(figsize=(len(gauges)*5,11))

for Gauge in order_colnames: 
    #Gauge = order_colnames[0]
    ax=plt.subplot(1,len(gauges),i) 
    plt.title( Gauge + ' water level density comparison '+ startdate + '-' + enddate)
    xlocation = 0.6
    Gauge_only_df = Histogram_df.filter(regex=Gauge[2:])
    combined_df.columns
    #scenario 1
    TS_1 = Gauge_only_df[Gauge+'_MHL'].iloc[:,0]
    if len(TS_1[TS_1.notnull()])>0:
        color_1 = 'royalblue'
        width = 1
        #full period density comparison for individual gauges overlaid by tidal planes 
        seaborn.kdeplot(TS_1, vertical=True, color=color_1, lw=2, ax=ax)
        #seaborn.kdeplot(Gauge_only_df.iloc[:,2], vertical=True, color=color_1, lw=3)
        plt.axhline(y=np.nanpercentile(TS_1,87), color=color_1, linestyle='solid', lw=width, alpha=1) 
        plt.text( xlocation , (np.nanpercentile(TS_1,87)), '13% exceedance', ha='left', va='bottom')
        plt.axhline(y=np.nanpercentile(TS_1,93.6), color=color_1, linestyle='dotted', lw=width, alpha=1) 
        plt.text(xlocation , (np.nanpercentile(TS_1,93.6)), '6.4% exceedance', ha='left', va='bottom')
        plt.axhline(y=np.nanpercentile(TS_1,98.7), color=color_1, linestyle='dashed', lw=width, alpha=1)
        plt.text( xlocation ,np.nanpercentile(TS_1,98.7) , '1.3% exceedance', ha='left', va='bottom')
        plt.axhline(y=np.nanpercentile(TS_1,99.99), color=color_1, linestyle='dashdot', lw=width, alpha=1) 
        plt.text( xlocation ,np.nanpercentile(TS_1,99.99) , '0.01% exceedance', ha='left', va='bottom')
        plt.axhline(y=np.nanpercentile(TS_1,50), color=color_1, linestyle='solid', lw=width, alpha=1) 
        plt.text( xlocation , np.nanpercentile(TS_1,50), '50% exceedance', ha='left', va='bottom')
    
    #scenario 2
    #TS_1 =  Gauge_only_df.filter(regex='elev')
    TS_1 = Gauge_only_df.iloc[:,0][Gauge_only_df.iloc[:,0].notnull()]
    color_1 = 'red'
    width = 1
    seaborn.kdeplot(TS_1, vertical=True, color=color_1, lw=2, ax=ax)
    #plt.axhline(y=np.nanpercentile(TS_1,87), color=color_1, linestyle='solid', lw=width, alpha=1) 
    #plt.text( xlocation , (np.nanpercentile(TS_1,87)), '13% exceedance', ha='left', va='bottom')
    #plt.axhline(y=np.nanpercentile(TS_1,93.6), color=color_1, linestyle='dotted', lw=width, alpha=1) 
    #plt.text(xlocation , (np.nanpercentile(TS_1,93.6)), '6.4% exceedance', ha='left', va='bottom')
    #plt.axhline(y=np.nanpercentile(TS_1,98.7), color=color_1, linestyle='dashed', lw=width, alpha=1)
    #plt.text( xlocation ,np.nanpercentile(TS_1,98.7) , '1.3% exceedance', ha='left', va='bottom')
    #plt.axhline(y=np.nanpercentile(TS_1,99.99), color=color_1, linestyle='dashdot', lw=width, alpha=1) 
    #plt.text( xlocation ,np.nanpercentile(TS_1,99.99) , '0.01% exceedance', ha='left', va='bottom')
    #plt.axhline(y=np.nanpercentile(TS_1,50), color=color_1, linestyle='solid', lw=width, alpha=1) 
    #plt.text( xlocation , np.nanpercentile(TS_1,50), '50% exceedance', ha='left', va='bottom')
    
    #scenario 3
    TS_1 = Gauge_only_df.iloc[:,1][Gauge_only_df.iloc[:,1].notnull()]
    color_1 = 'limegreen'
    width = 1
    seaborn.kdeplot(TS_1, vertical=True, color=color_1, lw=2, ax=ax)
    plt.axhline(y=np.nanpercentile(TS_1,87), color=color_1, linestyle='solid', lw=width, alpha=1) 
    #plt.text( xlocation , (np.nanpercentile(TS_1,87)), '13% exceedance', ha='left', va='bottom')
    plt.axhline(y=np.nanpercentile(TS_1,93.6), color=color_1, linestyle='dotted', lw=width, alpha=1) 
    #plt.text(xlocation , (np.nanpercentile(TS_1,93.6)), '6.4% exceedance', ha='left', va='bottom')
    plt.axhline(y=np.nanpercentile(TS_1,98.7), color=color_1, linestyle='dashed', lw=width, alpha=1)
    #plt.text( xlocation ,np.nanpercentile(TS_1,98.7) , '1.3% exceedance', ha='left', va='bottom')
    plt.axhline(y=np.nanpercentile(TS_1,99.99), color=color_1, linestyle='dashdot', lw=width, alpha=1) 
    #plt.text( xlocation ,np.nanpercentile(TS_1,99.99) , '0.01% exceedance', ha='left', va='bottom')
    plt.axhline(y=np.nanpercentile(TS_1,50), color=color_1, linestyle='solid', lw=width, alpha=1) 
    #plt.text( xlocation , np.nanpercentile(TS_1,50), '50% exceedance', ha='left', va='bottom')  
    
    #finish the plot
    plt.ylim(ylims)
    if i == 1: 
        plt.ylabel("Water level [m AHD]")
        
    plt.xlabel("Probability/density")
    fig.tight_layout()
    i=i+1
fig.savefig(png_out_name)
plt.close()
#========================================================#   


#========================================================#   
# Plot MHL water levels only
#========================================================#   
ylims = [-1,1.5]
#set plot parameters
ALPHA_figs = 1
font = {'family' : 'sans-serif',
        'weight' : 'normal',
        'size'   : 6}
matplotlib.rc('font', **font)

i=1
png_out_name = output_directory + '/' + Plot_scenario + '/MHL_gauges_water level histograms and exceedance planes '+ startdate + '_' + enddate + '.pdf' 
fig = plt.figure(figsize=(len(gauges)*5,11))

for Gauge in order_colnames: 
    #Gauge = order_colnames[0]
    ax=plt.subplot(1,len(gauges),i) 
    plt.title( Gauge + ' water level density comparison '+ startdate + '-' + enddate)
    xlocation = 0.6
    TS_1 = combined_df[Gauge+'_MHL'].iloc[:,0]
    TS_1 = TS_1[TS_1.notnull()]
    #scenario 1
    TS_1.name = 'Full Period ' + str(TS_1.index[0].date()) + '_' + str(TS_1.index[-1].date())
    color_1 = 'royalblue'
    width = 1
    #full period density comparison for individual gauges overlaid by tidal planes 
    seaborn.kdeplot(TS_1, vertical=True, color=color_1, lw=2, ax=ax)
    #seaborn.kdeplot(Gauge_only_df.iloc[:,2], vertical=True, color=color_1, lw=3)
    plt.axhline(y=np.nanpercentile(TS_1,87), color=color_1, linestyle='solid', lw=width, alpha=1) 
    plt.text( xlocation , (np.nanpercentile(TS_1,87)), '13% exceedance', ha='left', va='bottom')
    plt.axhline(y=np.nanpercentile(TS_1,93.6), color=color_1, linestyle='dotted', lw=width, alpha=1) 
    plt.text(xlocation , (np.nanpercentile(TS_1,93.6)), '6.4% exceedance', ha='left', va='bottom')
    plt.axhline(y=np.nanpercentile(TS_1,98.7), color=color_1, linestyle='dashed', lw=width, alpha=1)
    plt.text( xlocation ,np.nanpercentile(TS_1,98.7) , '1.3% exceedance', ha='left', va='bottom')
    plt.axhline(y=np.nanpercentile(TS_1,99.99), color=color_1, linestyle='dashdot', lw=width, alpha=1) 
    plt.text( xlocation ,np.nanpercentile(TS_1,99.99) , '0.01% exceedance', ha='left', va='bottom')
    plt.axhline(y=np.nanpercentile(TS_1,50), color=color_1, linestyle='solid', lw=width, alpha=1) 
    plt.text( xlocation , np.nanpercentile(TS_1,50), '50% exceedance', ha='left', va='bottom')
    
    #scenario 2
    #TS_1 =  Gauge_only_df.filter(regex='elev')

    TS_1 = TS_1.loc[datetime.strptime(startdate , '%Y %m %d').date()  : datetime.strptime(enddate , '%Y %m %d').date()]
    if len(TS_1[TS_1.notnull()])>0:
        TS_1.name = 'Period from ' + startdate + '_' + enddate
        color_1 = 'red'
        width = 1
        seaborn.kdeplot(TS_1, vertical=True, color=color_1, lw=2, ax=ax)
        plt.axhline(y=np.nanpercentile(TS_1,87), color=color_1, linestyle='solid', lw=width, alpha=1) 
        plt.text( xlocation , (np.nanpercentile(TS_1,87)), '13% exceedance', ha='left', va='bottom')
        plt.axhline(y=np.nanpercentile(TS_1,93.6), color=color_1, linestyle='dotted', lw=width, alpha=1) 
        plt.text(xlocation , (np.nanpercentile(TS_1,93.6)), '6.4% exceedance', ha='left', va='bottom')
        plt.axhline(y=np.nanpercentile(TS_1,98.7), color=color_1, linestyle='dashed', lw=width, alpha=1)
        plt.text( xlocation ,np.nanpercentile(TS_1,98.7) , '1.3% exceedance', ha='left', va='bottom')
        plt.axhline(y=np.nanpercentile(TS_1,99.99), color=color_1, linestyle='dashdot', lw=width, alpha=1) 
        plt.text( xlocation ,np.nanpercentile(TS_1,99.99) , '0.01% exceedance', ha='left', va='bottom')
        plt.axhline(y=np.nanpercentile(TS_1,50), color=color_1, linestyle='solid', lw=width, alpha=1) 
        plt.text( xlocation , np.nanpercentile(TS_1,50), '50% exceedance', ha='left', va='bottom')
        
    #finish the plot
    plt.ylim(ylims)
    if i == 1: 
        plt.ylabel("Water level [m AHD]")
        
    plt.xlabel("Probability/density")
    fig.tight_layout()
    i=i+1
fig.savefig(png_out_name)
plt.close()
#========================================================#   


#========================================================#   
# Plot time series over various intervals at control points
#========================================================#  
#set plot parameters
ALPHA_figs = 1
font = {'family' : 'sans-serif',
        'weight' : 'normal',
        'size'   : 5}
matplotlib.rc('font', **font)

halfwindows = [1,2,5,10]
for halfwindow in halfwindows:
    #halfwindow = 10
    png_out_name = output_directory + '/' + Plot_scenario +  '/water level time series '+ startdate + '_' + enddate + ' _window_of_' + str(halfwindow*2) +'_days.pdf' 
    fig = plt.figure(figsize=(10,len(gauges)*3))
    i=1
    for Gauge in order_colnames:
        Gauge_only_df = combined_df.filter(regex=Gauge[2:])
        #Gauge_only_df = df1.filter(regex=Gauge) 
        #Gauge_only_df_1 = Gauge_only_df.loc[(Gauge_only_df.index >= '2008-05-09 12:00:00') & (Gauge_only_df.index <= '2008-05-20 12:00:00')]
        Gauge_only_df_1 = Gauge_only_df[datetime.strptime(tideanalysis_day , '%Y %m %d').date()- timedelta(days=halfwindow) : datetime.strptime(tideanalysis_day , '%Y %m %d').date() + timedelta(days=halfwindow) ]
        Gauge_only_df_1 = Gauge_only_df_1[Gauge_only_df_1.iloc[:,0].notnull()]
        ax=plt.subplot(len(gauges),1,i) 
        plt.title('Tidal water levels at ' + Gauge + ' | ' + River +' Estuary') 
        ax.set_prop_cycle(color=['steelblue',  'coral', 'steelblue', 'deeppink'],linestyle=[ '-', '-','--','-'],   lw=[0.8,0.8,0.8,0.8])
        Gauge_only_df_1.iloc[:,[0,1,2]].plot( legend=False, ax=ax)

        plt.axhline(y=0, color='black', linestyle='-', lw=0.8, alpha=0.5)  
        #plt.axhline(y=0.9, color='tomato', linestyle='--', lw=1, alpha=0.5) 
        plt.ylim(ylims)
        plt.ylabel("Water level [m AHD]")
        plt.xlabel("Time")
        ax.legend()                     
        fig.tight_layout()
        i=i+1
    fig.savefig(png_out_name)
    plt.close()   
#========================================================#   
# Plot water level histograms at control points
#========================================================#  












##========================================================#   
##write out exceedance data to csv
##========================================================#   
#Summary={} 
#Summary['13% exceedance'] = Wollow_13ExPr 
#Summary['6.4% exceedance'] = Wollow_6p4ExPr
#Summary['1.3% exceedance'] = Wollow_1_3ExPr 
#Summary['0.01% exceedance'] = Wollow_0p01ExPr 
#Summary['50% exceedance'] = Wollow_50ExPr 
#pdf1 = pd.DataFrame(Summary, index=[0])
#pdf1.to_csv(os.path.join( Out_Folder + Gauge +  '_entrance_stats.csv'))
##========================================================#   
#
#
#
#
#
#
#
#
#
#
#
#
#
#
#
#
#
#
#
#
#
#
#
#
#
#
#
#
#
#
#
#
#
#
#
#
#
#
#
#
#
#
##Create Bathymetry along the nodes and chainages
#Elev_df = HD_Summary_df.filter(regex='Hwq003').filter(regex='elev')
#Elev_df.columns = chainages
#Depth_df = HD_Summary_df.filter(regex='Hwq003').filter(regex='depth')
#Depth_df.columns = chainages
#Bathy_df = pd.DataFrame(Elev_df.loc[maxtide_index] -  Depth_df.loc[maxtide_index])
#Bathy_df.columns = ['Bathymetry']
#Bathy_df.plot()
#
#bathy_out_path = 'H:/WRL_Projects/Hunter_CC_Modeling/Module_6/03_Results/Chainages/Hunter_nodes_bathy.csv'
#Bathy_df.to_csv(bathy_out_path)    
##plt.show()
#
#
##==========================================================#
##Tidal analysis
##==========================================================#
##read in tide data
#tides_df = tides_dfm[datetime.strptime(tideanalysis_day , '%Y %m %d').date()- timedelta(days=2) : datetime.strptime(tideanalysis_day , '%Y %m %d').date() + timedelta(days=3) ]
#central_tides_df = tides_dfm[datetime.strptime(tideanalysis_day , '%Y %m %d').date():datetime.strptime(tideanalysis_day , '%Y %m %d').date()]
#
#maxtide_index = central_tides_df['tide'].idxmax()
#end_maxtide_index =  maxtide_index + timedelta(days=0.4)
#beg_maxtide_index =  maxtide_index - timedelta(days=0.2)
#short_tides_df = tides_df[beg_maxtide_index: end_maxtide_index]
#short_tides_df.plot(cmap='viridis',  legend=True)
#mintide_index = short_tides_df['tide'].idxmin()
#
##plot the minimum and maximum elevation along the estuary occuring within a single tidal cycle
#Short_tide_elev_df = HD_Summary_df.filter(regex='elev')[min(short_tides_df.index):max(short_tides_df.index)]
#max_min_1cycle_elev_0slr = pd.concat([Short_tide_elev_df.filter(regex=fs[0]).max(), Short_tide_elev_df.filter(regex=fs[0]).min()], axis=1, join='inner')  
#max_min_1cycle_elev_09slr = pd.concat([Short_tide_elev_df.filter(regex=fs[1]).max(), Short_tide_elev_df.filter(regex=fs[1]).min()], axis=1, join='inner')
#max_min_1cycle_elev_0slr.index = chainages
#max_min_1cycle_elev_09slr.index = chainages
#max_min_1cycle_elev = pd.concat([max_min_1cycle_elev_0slr, max_min_1cycle_elev_09slr], axis=1, join='outer')
#max_min_1cycle_elev.columns = ['MaxElev 0 SLR','MinElev 0 SLR','MaxElev 0.9 SLR','MinElev 0.9 SLR' ]
#max_min_1cycle_elev.plot(cmap='viridis',  legend=True)
#
##plot the elevation along the estuary at exactly the time of high and low tide
#max_min_extime_elev_0slr = pd.concat([HD_Summary_df.filter(regex=fs[0]).filter(regex='elev').loc[maxtide_index], Summary_df.filter(regex=fs[0]).loc[mintide_index]], axis=1, join='outer')
#max_min_extime_elev_09slr = pd.concat([HD_Summary_df.filter(regex=fs[1]).filter(regex='elev').loc[maxtide_index], Summary_df.filter(regex=fs[1]).loc[mintide_index]], axis=1, join='outer')
#max_min_extime_elev_0slr.index = chainages
#max_min_extime_elev_09slr.index = chainages
#max_min_extime_elev = pd.concat([max_min_extime_elev_0slr, max_min_extime_elev_09slr], axis=1, join='outer')
#max_min_extime_elev.columns = ['HigT Elev 0SLR','LowT Elev 0SLR','HigT Elev 0.9 SLR','LowT Elev 0.9 SLR' ]
#max_min_extime_elev.plot(cmap='viridis',  legend=True)
#
#png_out_name = output_directory +'/' + Plot_scenario + '/' + variable + '_high_low_tide_analysis_' + Plot_scenario + '.pdf' 
#fig = plt.figure(figsize=(17,20))
#ax=plt.subplot(3,1,1) 
#plt.title('Max and min water surface elevation along the estuary over one tidal cycle') 
#ax.set_prop_cycle(color=['slateblue', 'slateblue', 'tomato', 'tomato'], linestyle=['--', '-', '--', '-'], lw=[2,2,2,2])
#max_min_1cycle_elev.plot( legend=False, ax=ax)
#plt.axhline(y=0, color='slateblue', linestyle='--', lw=1, alpha=0.5)  
#plt.axhline(y=0.9, color='tomato', linestyle='--', lw=1, alpha=0.5) 
#plt.ylim(ylims)
#plt.ylabel("Water level [m AHD]")
#plt.xlabel("Distance from ocean boundary [km]")
## Put a legend to the right of the current axis
##lgd = ax.legend(loc='center left', bbox_to_anchor=(1, 0.5))  
#ax.legend()                    
##plot tides
#ax=plt.subplot(3,1,2)
#plt.title('Max and min water surface elevation along the estuary at time of high & low tide') 
#ax.set_prop_cycle(color=['slateblue', 'slateblue', 'tomato', 'tomato'], linestyle=['--', '-', '--', '-'], lw=[2,2,2,2])
#max_min_extime_elev.plot(legend=False, ax=ax)
#plt.ylabel("Water level [m AHD]")
#plt.ylim(ylims)
#plt.xlabel("Distance from ocean boundary [km]")
##lgd = ax.legend(loc='center left', bbox_to_anchor=(1, 0.5))   
#ax.legend()    
##plot tides
#ax=plt.subplot(3,1,3)
#plt.ylim(ylims)
#plt.title('Tide at open ocean boundary') 
#short_tides_df.plot(cmap='viridis',  legend=False, ax=ax)
#ax.axvline(maxtide_index, color='black', linestyle='--')
#ax.axvline(mintide_index, color='black', linestyle='-')
#plt.xlabel("Time")
#plt.ylabel("Water level [m AHD]")
##lgd = ax.legend(loc='center left', bbox_to_anchor=(1, 0.5)) 
#ax.legend()     
#fig.tight_layout()
#fig.savefig(png_out_name)
#plt.close()
#
#
##==========================================================#
##animate 2 tidal cycles baseline scenario:
##==========================================================#
#Six_day_tide_elev_df = HD_Summary_df.filter(regex=fs[0]).filter(regex='elev')[maxtide_index - timedelta(days=2): maxtide_index + timedelta(days=3)]
#Six_day_tide_elev_df.columns = chainages
#Six_day_tides_df = tides_df[maxtide_index - timedelta(days=2): maxtide_index + timedelta(days=3)]
#Six_day_tides_df.plot(cmap='viridis',  legend=True)
#
#Six_day_flow_df = Flow_df[maxtide_index - timedelta(days=2): maxtide_index + timedelta(days=3)]
#Six_day_flow_df = pd.concat([Six_day_tides_df,  Six_day_flow_df], axis=1, join='outer')['Q[ML/d]'].interpolate()
#Six_day_flow_df.plot(cmap='viridis',  legend=True)
#
#ii=0
#for index in Six_day_tide_elev_df.index[0::4]:
#    ii=ii+1
#    #index = Six_day_tide_elev_df.index[1]
#    png_out_path =  output_directory +  '/' + Plot_scenario +  '/TidalCycle_' + variable + '_' + Plot_scenario +  '/'
#    if not os.path.exists(png_out_path):
#        os.makedirs(png_out_path)
#    #png_out_name = png_out_path + str(index) + 'longitudinalXS.png' 
#    png_out_name = png_out_path + str(ii) + variable +  '_longitudinalXS.png' 
#    fig = plt.figure(figsize=(17,20))
#    ax=plt.subplot(3,1,1) 
#    plt.title('Water surface elevation along the estuary') 
#    Six_day_tide_elev_df.loc[index].plot(color='blue',  legend=False)
#    #plt.axhline(y=0, color='r', linestyle='--', lw=1, alpha=0.5) 
#    plt.axhline(tides_df.loc[index].values, color='black',lw=0.5, linestyle='--')
#    plt.ylim(ylims)
#    plt.ylabel("Water level [m AHD]")
#    plt.xlabel("Distance from ocean boundary [km]")
#    #plot tides
#    ax=plt.subplot(3,1,2)
#    plt.ylim(-1.1,1.3)
#    plt.title('Tide at open ocean boundary') 
#    Six_day_tides_df.plot(cmap='viridis',  legend=False, ax=ax)
#    ax.axvline(index, color='green', linestyle='--')
#    plt.xlabel("Time")
#    plt.ylabel("Water level [m AHD]")
#    
#    ax=plt.subplot(3,1,3)
#    #plt.ylim(ylims)
#    plt.title('Flow at the upstream river boundary') 
#    Six_day_flow_df.plot(color='black',  legend=False, ax=ax)
#    ax.axvline(index, color='green', linestyle='--')
#    plt.xlabel("Time")
#    plt.ylabel("Hunter Inflow [ML/day]")
#
#    fig.tight_layout()
#    fig.savefig(png_out_name)
#    plt.close()
#
#        
#        
#
#
#
##==========================================================#
##animate 2 tidal cycles baseline + SLR scenario:
##==========================================================#
#Six_day_tide_elev_df_0slr = HD_Summary_df.filter(regex=fs[0]).filter(regex='elev')[maxtide_index - timedelta(days=1): maxtide_index + timedelta(days=1)]
#Six_day_tide_elev_df_0slr.columns = chainages
#Six_day_tide_elev_df_09slr = HD_Summary_df.filter(regex=fs[1]).filter(regex='elev')[maxtide_index - timedelta(days=1): maxtide_index + timedelta(days=1)]
#Six_day_tide_elev_df_09slr.columns = chainages
#Six_day_tides_df = tides_df[maxtide_index - timedelta(days=1): maxtide_index + timedelta(days=1.5)]
#Six_day_tides_df.plot(cmap='viridis',  legend=True)
#
#ii=0
#for index in Six_day_tide_elev_df_0slr.index:
#    ii=ii+1
#    #index = Six_day_tide_elev_df_0slr.index[1]
#    png_out_path =  output_directory + '/' + Plot_scenario +   '/TidalCycle_SLR_' + variable + '_' + Plot_scenario +   '/'
#    if not os.path.exists(png_out_path):
#        os.makedirs(png_out_path)
#    
#    #png_out_name = png_out_path + str(index) + 'longitudinalXS.png' 
#    png_out_name = png_out_path + str(ii) + variable +  '_longitudinalXS.png' 
#    max_min_extime_elev_both = pd.concat([Six_day_tide_elev_df_0slr.loc[index],  Six_day_tide_elev_df_09slr.loc[index]], axis=1, join='outer')
#    max_min_extime_elev_both.columns = ['Present Day','0.9m SLR']
#    
#    fig = plt.figure(figsize=(17,12))
#    ax=plt.subplot(2,1,1) 
#    plt.ylim(ylims)
#    plt.title('Water surface elevation along the estuary') 
#    ax.set_prop_cycle(color=['slateblue','tomato'], linestyle=['-',  '-'], lw=[2,2])
#    max_min_extime_elev_both.plot(ax=ax)
#    ax.legend(loc=1)
#    plt.axhline(y=0, color='r', linestyle='--', lw=1, alpha=0.5)  
#    plt.ylim(-1.1,2.5)
#    plt.ylabel("Water level [m AHD]")
#    plt.xlabel("Distance from ocean boundary [km]")
#    
#    #plot tides
#    ax=plt.subplot(2,1,2)
#    plt.ylim(-1.1,1.5)
#    plt.title('Tide at open ocean boundary') 
#    Six_day_tides_df.plot(cmap='viridis',  legend=False, ax=ax)
#    ax.axvline(index, color='green', linestyle='--')
#    plt.xlabel("Time")
#    plt.ylabel("Water level [m AHD]")
#    fig.tight_layout()
#    fig.savefig(png_out_name)
    plt.close()
 
        
































#for NODE in node:
#    NODE = str(NODE)
#    #=========#
#    #comparison time series plot at node
#    #=========#
#    #out name
#    png_out_file_name = comparison_code + '_' +  variable  + '_' + NODE + '_time_series.png'
#    png_out_path = png_output_directory + '/' + png_out_file_name
#    #
#    plotin_df = Summary_df.filter(regex=NODE)
#    #prepare data frame for plot
#    fig = plt.figure(figsize=(14,18))
#    ax=plt.subplot(4,1,1) 
#    #start and end dat of plots
#    plt.title(variable  + '_' + NODE + ' - time series')
#    start_date = datetime(int('1996'), int('02'), int('01'))
#    end_date = datetime(int('1996'), int('02'), int('5'))
#    plotin_df1=plotin_df[start_date:end_date]
#    ymin = min(plotin_df1.min()) - 0.01 *min(plotin_df1.min())
#    ymax = max(plotin_df1.max()) + 0.01 *max(plotin_df1.max())  
#    plotin_df1.plot(ylim=(ymin,ymax) ,legend=False, ax=ax) 
#    #plt.legend( loc=1)    
#    plt.xticks([])
#    
#    ax=plt.subplot(4,1,2) 
#    start_date = datetime(int('1996'), int('02'), int('01'))
#    end_date = datetime(int('1996'), int('02'), int('3'))
#    plotin_df2=plotin_df[start_date:end_date]
#    ymin = min(plotin_df2.min()) - 0.1 *min(plotin_df2.min())
#    ymax = max(plotin_df2.max()) + 0.1 *max(plotin_df2.max()) 
#    plotin_df2.plot(ylim=(ymin,ymax),legend=False, ax=ax)
#    #plt.legend( loc=1)
#    ax.xaxis.grid(False)
#    
#    ax=plt.subplot(4,1,3)
#    plt.title(variable  + '_' + NODE + ' - monthly maximum')
#    Monthly_max = plotin_df.resample('M').max()
#    Monthly_max.plot(ax=ax,legend=False)
#    plt.legend( loc=1)
#    ax.xaxis.grid(False)
#    
#    ax=plt.subplot(4,1,4)
#    plt.title(variable  + '_' + NODE + ' - monthly mean')
#    Monthly_mean = plotin_df.resample('M').mean()
#    Monthly_mean.plot(ax=ax)
#    #plt.legend(bbox_to_anchor=(0., 1.02, 1., .102), loc=3,
#    #   ncol=2, mode="expand", borderaxespad=0.)
#    #plt.legend( loc=1)
#    ax.xaxis.grid(False)
#
#    #ax.patch.set_alpha(ALPHA_figs)
#    fig.patch.set_alpha(ALPHA_figs)
#    fig.tight_layout()
#    fig.savefig(png_out_path)
#    plt.close()
#    #=========#



